#pragma once
#include <SDL_events.h>
#include <unordered_map>
#include <vector>

#include "GamePad.h"
#include "KActionBind.h"

namespace KapitanGame {
	class KInput {
		std::vector<SDL_GameController*> ConnectedControllers;
		std::vector<GamePad> ControllerInputs;
		std::vector<GamePad> LastControllerInputs;
		std::vector<Uint8> KeyboardInputs;
		std::vector<Uint8> LastKeyboardInputs;
		std::vector<KActionBind> Actions;
		std::unordered_map<std::string, Uint8> KeyBinds;
		int GamepadsCount;
		bool Initialized;
	public:
		KInput();
		void Init();
		void Free();
		void HandleInputPreEvents();
		void HandleInputPostEvents();
		void HandleEvent(const SDL_Event& event);
		void BindAction(std::string name, InputState expectedInputState, KPlayerController* controller, KPlayerCommand command);
		bool CheckAction(const std::string& name, InputState expectedInputState, Controllers controller);
		void ProcessActions();
		bool IsControllerButtonPressed(Controllers controllerId, SDL_GameControllerButton button);
		bool IsControllerButtonReleased(Controllers controllerId, SDL_GameControllerButton button);
		bool IsControllerButtonHeld(Controllers controllerId, SDL_GameControllerButton button);
		float GetControllerAxis(Controllers controllerId, SDL_GameControllerAxis axis);
		bool IsKeyboardButtonHeld(Uint8 scanCode);
		bool IsKeyboardButtonPressed(Uint8 scanCode);
		bool IsKeyboardButtonReleased(Uint8 scanCode);
	};
}


